@@ -93,7 +93,7 @@ def _unnest_column(df: DataFrame, col: str):
         return df
 
 
-def _analyze_column(df: DataFrame, schema_field: SchemaField):
+def _analyze_column(df: DataFrame, schema_field: SchemaField, col_num: int):
     col = schema_field.name
     if "!" in col:
         if col[-1] == "!":
@@ -102,6 +102,7 @@ def _analyze_column(df: DataFrame, schema_field: SchemaField):
             col = col.split(".")[-1]
     df = _unnest_column(df, schema_field.name)
     res = df.select(
+        f.lit(col_num).alias("column_number"),
         f.lit(schema_field.name).alias("column_name"),
         f.lit(schema_field.field_type).alias("column_type"),
         f.count(f.lit(1)).alias("count"),
@@ -114,7 +115,13 @@ def _analyze_column(df: DataFrame, schema_field: SchemaField):
     return res
 
 
-def analyze(df: DataFrame):
+def __chunks(lst: List, n: int):
+    """Yield successive n-sized chunks from lst."""
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
+
+
+def analyze(df: DataFrame, _chunk_size: int = 50):
     """Analyze a DataFrame by computing various stats for each column.
 
     It returns a DataFrame with one row per column and the following columns:
@@ -136,6 +143,20 @@ def analyze(df: DataFrame):
     - Arrays are unnested with a `!` character in their name, which is why they may have a different count.
     - Null values are not counted in the count_distinct column.
 
+    Troubleshooting:
+    -----------------------
+    When working on entreprise-level data, BigQuery might return a 400 error with the following message.
+    "Not enough resources for query planning - too many subqueries or query is too complex"
+
+    This may happen in two cases:
+    - If the input DataFrame is already the result of a complex calculation, it is recommended to save it to a
+      permanent or temporary table first. You can save it to a temporary table by using `df = df.persist()`
+    - When analyzing DataFrames with hundreds of columns, the query generated by analyze() might get too complex
+      for BigQuery. To avoid this, we split the query into smaller parts that are persisted.
+      The optional argument `_chunk_size` (default = 50) determines the number of columns analyzed per chunk.
+      If you already persisted the input DataFrame and the same error persist, you can try lowering the value
+      of `_chunk_size`, but beware that the lower this value, the slower the analyzis will be.
+
     >>> df = __get_test_df()
     >>> df.show()
     +----+------------+---------------------+--------------------------------------------+
@@ -152,6 +173,7 @@ def analyze(df: DataFrame):
     | 9  | Blastoise  |      ['Water']      |  {'can_evolve': False, 'evolves_from': 8}  |
     +----+------------+---------------------+--------------------------------------------+
     >>> df = analyze(df)
+    Analyzing 5 columns ...
     >>> df.withColumn("approx_top_100", f.expr("approx_top_100[OFFSET(0)]"), replace=True).show()
     +------------------------+-------------+-------+----------------+------------+-----------+-----------+------------------------------------+
     |      column_name       | column_type | count | count_distinct | count_null |    min    |    max    |           approx_top_100           |
@@ -164,11 +186,27 @@ def analyze(df: DataFrame):
     +------------------------+-------------+-------+----------------+------------+-----------+-----------+------------------------------------+
 
     :param df:
+    :param _chunk_size: (default: 50) Try reducing this number if you get this error from BigQuery: "Not enough resources for query planning"
     :return:
     """
     flat_schema = flatten_schema(df.schema, explode=True)
-    col_dfs = [_analyze_column(df, schema_field) for schema_field in flat_schema]
-    return union_dataframes(col_dfs)
+    col_dfs = [_analyze_column(df, schema_field, num) for num, schema_field in enumerate(flat_schema)]
+
+    nb_cols = len(col_dfs)
+    print(f"Analyzing {nb_cols} columns ...")
+    if nb_cols <= _chunk_size:
+        union_df = union_dataframes(col_dfs)
+    else:
+        from tqdm import tqdm
+        big_dfs = [
+            union_dataframes(chunk).persist()
+            for chunk in tqdm(list(__chunks(col_dfs, _chunk_size)))
+        ]
+        union_df = union_dataframes(big_dfs)
+    # For some reason, `union_df.orderBy("column_number").drop("column_number")`
+    # does not preserve the ordering, but this does:
+    query = f"""SELECT * EXCEPT(column_number) FROM {quote(union_df._alias)} ORDER BY column_number"""
+    return union_df._apply_query(query)
 
 
 def __get_test_df() -> DataFrame:
@@ -216,4 +254,3 @@ def __get_test_df() -> DataFrame:
         ])
     """
     return bq.sql(query)
-
